'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

require('@babel/runtime/helpers/objectWithoutPropertiesLoose');
require('@babel/runtime/helpers/extends');
var css = require('css');
var serializer = require('./serializer-3f99148d.cjs.dev.js');
var chalk = _interopDefault(require('chalk'));
var specificity = require('specificity');

/*
 * Taken from
 * https://github.com/facebook/jest/blob/be4bec387d90ac8d6a7596be88bf8e4994bc3ed9/packages/expect/src/jasmine_utils.js#L234
 */

function isA(typeName, value) {
  return Object.prototype.toString.apply(value) === "[object " + typeName + "]";
}
/*
 * Taken from
 * https://github.com/facebook/jest/blob/be4bec387d90ac8d6a7596be88bf8e4994bc3ed9/packages/expect/src/jasmine_utils.js#L36
 */


function isAsymmetric(obj) {
  return obj && isA('Function', obj.asymmetricMatch);
}

function valueMatches(declaration, value) {
  if (value instanceof RegExp) {
    return value.test(declaration.value);
  }

  if (isAsymmetric(value)) {
    return value.asymmetricMatch(declaration.value);
  }

  return value === declaration.value;
}

function toHaveStyleRule(received, property, value, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      target = _options.target,
      media = _options.media;
  var classNames = serializer.getClassNamesFromNodes([received]);
  var cssString = serializer.getStylesFromClassNames(classNames, serializer.getStyleElements());
  var styles = css.parse(cssString);
  var preparedRules = styles.stylesheet.rules;

  if (media) {
    preparedRules = serializer.getMediaRules(preparedRules, media);
  }

  var result = preparedRules.filter(function (rule) {
    return rule.type === serializer.RULE_TYPES.rule && serializer.hasClassNames(classNames, rule.selectors, target);
  }).reduce(function (acc, rule) {
    var lastMatchingDeclaration = serializer.findLast(rule.declarations, function (dec) {
      return dec.type === 'declaration' && dec.property === property;
    });

    if (!lastMatchingDeclaration) {
      return acc;
    }

    return acc.concat(rule.selectors.map(function (selector) {
      return {
        selector: selector,
        declaration: lastMatchingDeclaration
      };
    }));
  }, []).sort(function (_ref, _ref2) {
    var selectorA = _ref.selector;
    var selectorB = _ref2.selector;
    return specificity.compare(selectorA, selectorB);
  }).pop();

  if (!result) {
    return {
      pass: false,
      message: function message() {
        return "Property not found: " + property;
      }
    };
  }

  var declaration = result.declaration;
  var pass = valueMatches(declaration, value);

  var message = function message() {
    return "Expected " + property + (pass ? ' not ' : ' ') + "to match:\n" + ("  " + chalk.green(value) + "\n") + 'Received:\n' + ("  " + chalk.red(declaration.value));
  };

  return {
    pass: pass,
    message: message
  };
}

var matchers = {
  toHaveStyleRule: toHaveStyleRule
};

var index = {
  print: serializer.print,
  test: serializer.test
};

exports.createSerializer = serializer.createSerializer;
exports.print = serializer.print;
exports.test = serializer.test;
exports.default = index;
exports.matchers = matchers;
