"use strict";

function _interopDefault(ex) {
  return ex && "object" == typeof ex && "default" in ex ? ex.default : ex;
}

var _objectWithoutPropertiesLoose = _interopDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose")), _extends = _interopDefault(require("@babel/runtime/helpers/extends")), css = require("css");

function defaultClassNameReplacer(className, index) {
  return "emotion-" + index;
}

var componentSelectorClassNamePattern = /^e[a-zA-Z0-9]+[0-9]+$/, replaceClassNames = function(classNames, styles, code, keys, classNameReplacer) {
  void 0 === classNameReplacer && (classNameReplacer = defaultClassNameReplacer);
  var index = 0, keyPattern = new RegExp("^(" + keys.join("|") + ")-");
  return classNames.reduce((function(acc, className) {
    if (keyPattern.test(className) || componentSelectorClassNamePattern.test(className)) {
      var escapedRegex = new RegExp(className.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&"), "g");
      return acc.replace(escapedRegex, classNameReplacer(className, index++));
    }
    return acc;
  }), styles + (styles ? "\n\n" : "") + code);
}, isBrowser = "undefined" != typeof document;

function last(arr) {
  return arr.length > 0 ? arr[arr.length - 1] : void 0;
}

function flatMap(arr, iteratee) {
  var _ref;
  return (_ref = []).concat.apply(_ref, arr.map(iteratee));
}

function findLast(arr, predicate) {
  for (var i = arr.length - 1; i >= 0; i--) if (predicate(arr[i])) return arr[i];
}

var RULE_TYPES = {
  media: "media",
  rule: "rule"
};

function getClassNames(selectors, classes) {
  return classes ? selectors.concat(classes.split(" ")) : selectors;
}

function getClassNamesFromTestRenderer(selectors, _ref2) {
  var _ref2$props = _ref2.props, props = void 0 === _ref2$props ? {} : _ref2$props;
  return getClassNames(selectors, props.className || props.class);
}

function shouldDive(node) {
  return "function" == typeof node.dive && "string" != typeof node.type();
}

function isTagWithClassName(node) {
  return node.prop("className") && "string" == typeof node.type();
}

function findNodeWithClassName(node) {
  var found = node.findWhere(isTagWithClassName);
  return found.length ? found.first() : null;
}

function getClassNameProp(node) {
  return node && node.prop("className") || "";
}

function getClassNamesFromEnzyme(selectors, node) {
  return getClassNames(selectors, getClassNameProp(findNodeWithClassName(shouldDive(node) ? node.dive() : node)));
}

function getClassNamesFromCheerio(selectors, node) {
  return getClassNames(selectors, node.attr("class"));
}

function getClassNamesFromDOMElement(selectors, node) {
  return getClassNames(selectors, node.getAttribute("class"));
}

function isReactElement(val) {
  return val.$$typeof === Symbol.for("react.test.json");
}

function isEmotionCssPropElementType(val) {
  return val.$$typeof === Symbol.for("react.element") && val.type.$$typeof === Symbol.for("react.forward_ref") && "EmotionCssPropInternal" === val.type.displayName;
}

function isEmotionCssPropEnzymeElement(val) {
  return val.$$typeof === Symbol.for("react.test.json") && "EmotionCssPropInternal" === val.type;
}

var domElementPattern = /^((HTML|SVG)\w*)?Element$/;

function isDOMElement(val) {
  return 1 === val.nodeType && val.constructor && val.constructor.name && domElementPattern.test(val.constructor.name);
}

function isEnzymeElement(val) {
  return "function" == typeof val.findWhere;
}

function isCheerioElement(val) {
  return "[cheerio object]" === val.cheerio;
}

function getClassNamesFromNodes(nodes) {
  return nodes.reduce((function(selectors, node) {
    return isReactElement(node) ? getClassNamesFromTestRenderer(selectors, node) : isEnzymeElement(node) ? getClassNamesFromEnzyme(selectors, node) : isCheerioElement(node) ? getClassNamesFromCheerio(selectors, node) : getClassNamesFromDOMElement(selectors, node);
  }), []);
}

var keyframesPattern = /^@keyframes\s+(animation-[^{\s]+)+/, removeCommentPattern = /\/\*[\s\S]*?\*\//g, getElementRules = function(element) {
  var nonSpeedyRule = element.textContent;
  return nonSpeedyRule ? [ nonSpeedyRule ] : element.sheet ? [].slice.call(element.sheet.cssRules).map((function(cssRule) {
    return cssRule.cssText;
  })) : [];
};

function getStylesFromClassNames(classNames, elements) {
  if (!classNames.length) return "";
  var keys = getKeys(elements);
  if (!keys.length) return "";
  var targetClassName = classNames.find((function(className) {
    return /^e[a-z0-9]+$/.test(className);
  })), keyPattern = "(" + keys.join("|") + ")-", classNamesRegExp = new RegExp(targetClassName ? "^(" + keyPattern + "|" + targetClassName + ")" : "^" + keyPattern), filteredClassNames = classNames.filter((function(className) {
    return classNamesRegExp.test(className);
  }));
  if (!filteredClassNames.length) return "";
  var selectorPattern = new RegExp("\\.(" + filteredClassNames.join("|") + ")"), keyframes = {}, styles = "";
  flatMap(elements, getElementRules).forEach((function(rule) {
    selectorPattern.test(rule) && (styles += rule);
    var match = rule.match(keyframesPattern);
    if (null !== match) {
      var name = match[1];
      void 0 === keyframes[name] && (keyframes[name] = ""), keyframes[name] += rule;
    }
  }));
  var keyframeNameKeys = Object.keys(keyframes), keyframesStyles = "";
  if (keyframeNameKeys.length) {
    var keyframesNamePattern = new RegExp(keyframeNameKeys.join("|"), "g"), keyframesNameCache = {}, index = 0;
    styles = styles.replace(keyframesNamePattern, (function(name) {
      return void 0 === keyframesNameCache[name] && (keyframesNameCache[name] = "animation-" + index++, 
      keyframesStyles += keyframes[name]), keyframesNameCache[name];
    })), keyframesStyles = keyframesStyles.replace(keyframesNamePattern, (function(value) {
      return keyframesNameCache[value];
    }));
  }
  return (keyframesStyles + styles).replace(removeCommentPattern, "");
}

function getStyleElements() {
  if (!isBrowser) throw new Error("jest-emotion requires jsdom. See https://jestjs.io/docs/en/configuration#testenvironment-string for more information.");
  return Array.from(document.querySelectorAll("style[data-emotion]"));
}

var unique = function(arr) {
  return Array.from(new Set(arr));
};

function getKeys(elements) {
  return unique(elements.map((function(element) {
    return element.getAttribute("data-emotion");
  }))).filter(Boolean);
}

function hasClassNames(classNames, selectors, target) {
  return selectors.some((function(selector) {
    if (!target) {
      var lastCls = last(selector.split(" "));
      return !!lastCls && classNames.includes(lastCls.slice(1));
    }
    return target instanceof RegExp ? target.test(selector) : selector.includes(target);
  }));
}

function getMediaRules(rules, media) {
  return rules.filter((function(rule) {
    var isMediaMatch = !!rule.media && rule.media.replace(/\s/g, "").includes(media.replace(/\s/g, ""));
    return rule.type === RULE_TYPES.media && isMediaMatch;
  })).reduce((function(mediaRules, mediaRule) {
    return mediaRules.concat(mediaRule.rules);
  }), []);
}

function isPrimitive(test) {
  return test !== Object(test);
}

function hasIntersection(left, right) {
  return left.some((function(value) {
    return right.includes(value);
  }));
}

function getNodes(node, nodes) {
  if (void 0 === nodes && (nodes = []), Array.isArray(node)) {
    var _iterator = node, _isArray = Array.isArray(_iterator), _i = 0;
    for (_iterator = _isArray ? _iterator : _iterator[Symbol.iterator](); ;) {
      var _ref;
      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        if ((_i = _iterator.next()).done) break;
        _ref = _i.value;
      }
      getNodes(_ref, nodes);
    }
    return nodes;
  }
  if (node.children) {
    var _iterator2 = node.children, _isArray2 = Array.isArray(_iterator2), _i2 = 0;
    for (_iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator](); ;) {
      var _ref2;
      if (_isArray2) {
        if (_i2 >= _iterator2.length) break;
        _ref2 = _iterator2[_i2++];
      } else {
        if ((_i2 = _iterator2.next()).done) break;
        _ref2 = _i2.value;
      }
      getNodes(_ref2, nodes);
    }
  }
  return "object" == typeof node && nodes.push(node), nodes;
}

function copyProps(src, target) {
  return Object.defineProperties(src, _extends({}, Object.getOwnPropertyDescriptors(target)));
}

function deepTransform(node, transform) {
  if (Array.isArray(node)) return node.map((function(child) {
    return deepTransform(child, transform);
  }));
  var transformed = transform(node);
  return transformed !== node && (transformed.props && copyProps(transformed, {
    props: Object.entries(transformed.props).reduce((function(props, _ref3) {
      var _Object$assign, key = _ref3[0], value = _ref3[1];
      return Object.assign(props, ((_Object$assign = {})[key] = deepTransform(value, transform), 
      _Object$assign));
    }), {})
  }), transformed.children) ? copyProps(transformed, {
    children: flatMap(deepTransform(transformed.children, transform), (function(id) {
      return id;
    }))
  }) : transformed;
}

function getPrettyStylesFromClassNames(classNames, elements) {
  var prettyStyles, styles = getStylesFromClassNames(classNames, elements);
  try {
    prettyStyles = css.stringify(css.parse(styles));
  } catch (e) {
    throw console.error(e), new Error('There was an error parsing the following css: "' + styles + '"');
  }
  return prettyStyles;
}

function filterEmotionProps(props) {
  void 0 === props && (props = {});
  var _props = props, rest = (_props.css, _props.__EMOTION_TYPE_PLEASE_DO_NOT_USE__, 
  _props.__EMOTION_LABEL_PLEASE_DO_NOT_USE__, _objectWithoutPropertiesLoose(_props, [ "css", "__EMOTION_TYPE_PLEASE_DO_NOT_USE__", "__EMOTION_LABEL_PLEASE_DO_NOT_USE__" ]));
  return rest.css = "unknown styles", rest;
}

function isShallowEnzymeElement(element, classNames) {
  return !hasIntersection(classNames, flatMap(element.children || [], (function(_ref4) {
    var _ref4$props = _ref4.props;
    return ((void 0 === _ref4$props ? {} : _ref4$props).className || "").split(" ");
  })).filter(Boolean));
}

var createConvertEmotionElements = function(keys, printer, isTransformed) {
  return function(node) {
    if (isTransformed(node) || isPrimitive(node)) return node;
    if (isEmotionCssPropEnzymeElement(node)) {
      var expectedClassNames = flatMap((node.props.css.name || "").split(" "), (function(cssClassName) {
        return keys.map((function(key) {
          return key + "-" + cssClassName;
        }));
      }));
      if (isShallowEnzymeElement(node, expectedClassNames)) {
        var _className = [ node.props.className ].concat(expectedClassNames).filter(Boolean).join(" "), emotionType = node.props.__EMOTION_TYPE_PLEASE_DO_NOT_USE__, type = "string" == typeof emotionType ? emotionType : emotionType.name;
        return _extends({}, node, {
          props: filterEmotionProps(_extends({}, node.props, {
            className: _className
          })),
          type: type
        });
      }
      return node.children;
    }
    return isEmotionCssPropElementType(node) ? _extends({}, node, {
      props: filterEmotionProps(node.props),
      type: node.props.__EMOTION_TYPE_PLEASE_DO_NOT_USE__
    }) : isReactElement(node) ? copyProps({}, node) : node;
  };
};

function clean(node, classNames) {
  if (Array.isArray(node)) {
    var _iterator3 = node, _isArray3 = Array.isArray(_iterator3), _i3 = 0;
    for (_iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator](); ;) {
      var _ref5;
      if (_isArray3) {
        if (_i3 >= _iterator3.length) break;
        _ref5 = _iterator3[_i3++];
      } else {
        if ((_i3 = _iterator3.next()).done) break;
        _ref5 = _i3.value;
      }
      clean(_ref5, classNames);
    }
  } else {
    if (node.children) {
      var _iterator4 = node.children, _isArray4 = Array.isArray(_iterator4), _i4 = 0;
      for (_iterator4 = _isArray4 ? _iterator4 : _iterator4[Symbol.iterator](); ;) {
        var _ref6;
        if (_isArray4) {
          if (_i4 >= _iterator4.length) break;
          _ref6 = _iterator4[_i4++];
        } else {
          if ((_i4 = _iterator4.next()).done) break;
          _ref6 = _i4.value;
        }
        clean(_ref6, classNames);
      }
    }
    if (node.props) {
      var _className2 = node.props.className;
      if (_className2) hasIntersection(_className2.split(" "), classNames) && delete node.props.css; else delete node.props.className;
    }
  }
}

function createSerializer(_temp) {
  var _ref7 = void 0 === _temp ? {} : _temp, classNameReplacer = _ref7.classNameReplacer, _ref7$DOMElements = _ref7.DOMElements, DOMElements = void 0 === _ref7$DOMElements || _ref7$DOMElements, cache = new WeakSet, isTransformed = function(node) {
    return cache.has(node);
  };
  return {
    test: function(val) {
      return val && !isTransformed(val) && (isReactElement(val) || DOMElements && isDOMElement(val));
    },
    print: function(val, printer) {
      var isNestedPrint = isTransformed(val), elements = getStyleElements(), keys = getKeys(elements), converted = deepTransform(val, createConvertEmotionElements(keys, printer, isTransformed)), nodes = getNodes(converted);
      nodes.forEach(cache.add, cache);
      var classNames = getClassNamesFromNodes(nodes), styles = getPrettyStylesFromClassNames(classNames, elements);
      clean(converted, classNames);
      var printedVal = printer(converted);
      return nodes.forEach(cache.delete, cache), isNestedPrint ? printedVal : replaceClassNames(classNames, styles, printedVal, keys, classNameReplacer);
    }
  };
}

var _createSerializer = createSerializer(), print = _createSerializer.print, test = _createSerializer.test;

exports.RULE_TYPES = RULE_TYPES, exports.createSerializer = createSerializer, exports.findLast = findLast, 
exports.getClassNamesFromNodes = getClassNamesFromNodes, exports.getMediaRules = getMediaRules, 
exports.getStyleElements = getStyleElements, exports.getStylesFromClassNames = getStylesFromClassNames, 
exports.hasClassNames = hasClassNames, exports.print = print, exports.test = test;
